
import numpy as np
import pandas as pd
data=pd.read_csv(r'hypothyroid.csv')
data.head()

data.info()

from sklearn.preprocessing import LabelEncoder
enc=LabelEncoder()
for x in data.columns:
    data[x]=enc.fit_transform(data[x])
data.info()

data.head()

data=data.drop_duplicates()
data.info()

data.describe()

data['age']=(data['age']-data['age'].min())/(data['age'].max()-data['age'].min())
data['TT4']=(data['TT4']-data['TT4'].min())/(data['TT4'].max()-data['TT4'].min())
data['T4U']=(data['T4U']-data['T4U'].min())/(data['T4U'].max()-data['T4U'].min())
data['FTI']=(data['FTI']-data['FTI'].min())/(data['FTI'].max()-data['FTI'].min())
data.head()

y=data['binaryClass']
x=data.drop(['binaryClass'],axis=1)

x.to_excel(excel_writer=r'test.xlsx')
y.to_excel(excel_writer=r'testy.xlsx')

from sklearn.model_selection import train_test_split
xtrain,xtest,ytrain,ytest= train_test_split(x,y,test_size=0.1,stratify=y)
print(xtrain.shape)
print(xtest.shape)
print(ytrain.shape)
print(ytest.shape)

#libraries
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
import plotly.offline as py
py.init_notebook_mode(connected=True)
import plotly.graph_objs as go
import plotly.tools as tls
#algorithms
from sklearn.ensemble import RandomForestClassifier, AdaBoostClassifier, GradientBoostingClassifier
from sklearn.discriminant_analysis import LinearDiscriminantAnalysis
from sklearn.linear_model import LogisticRegression
from sklearn.neighbors import KNeighborsClassifier
from sklearn.tree import DecisionTreeClassifier
from sklearn.svm import SVC
from sklearn.linear_model import SGDClassifier
#other
from sklearn.preprocessing import LabelEncoder
from sklearn.model_selection import train_test_split
from sklearn.model_selection import GridSearchCV, cross_val_score, StratifiedKFold, learning_curve, KFold
from sklearn.ensemble import StackingClassifier, VotingClassifier
#ignore warnings
import warnings
warnings.filterwarnings('ignore')

corrdata = data.corr()
ax,fig = plt.subplots(figsize=(15,8))
sns.heatmap(corrdata,annot=True)

from sklearn.model_selection import train_test_split

#split the data to train and test
X_train,X_test,y_train,y_test = train_test_split(x,y,test_size = 0.2,random_state=0)

# Cross validate model with Kfold stratified cross val
kfold = StratifiedKFold(n_splits=5)
#MODELING  WITH ADABOOST, RANDOM FOREST, SVM, GRADIENTBOOSTING and LOGISTIC REGRESSION

# Adaboost
DTC = DecisionTreeClassifier()
adaDTC = AdaBoostClassifier(DTC)

# Grid search for optimal parameters
ada_param_grid = {"base_estimator__criterion" : ["gini", "entropy"],
              "base_estimator__splitter" :   ["best", "random"],
              "algorithm" : ["SAMME","SAMME.R"],
              "n_estimators" :[1,2],
              "learning_rate":  [0.001, 0.01, 0.1, 0.3,1.5]}

gsadaDTC = GridSearchCV(adaDTC,param_grid = ada_param_grid, cv=kfold, scoring="accuracy", verbose = 1)

gsadaDTC.fit(X_train,y_train)

#model best estimator
ada_best = gsadaDTC.best_estimator_

#best score
gsadaDTC.best_score_

# Random Forest
RFC = RandomForestClassifier()

rf_param_grid = {"max_depth": [None,4],
              "max_features": [1, 3, 10],
              "min_samples_split": [2, 3, 10],
              "min_samples_leaf": [1, 3, 10],
              "bootstrap": [False],
              "n_estimators" :[100,300],
              "criterion": ["gini"]}


gsRFC = GridSearchCV(RFC,param_grid = rf_param_grid, cv=kfold, scoring="accuracy", verbose = 1)

gsRFC.fit(X_train,y_train)


rf_best = gsRFC.best_estimator_

gsRFC.best_score_

# Gradient Boosting 
GBC = GradientBoostingClassifier()
gb_param_grid = {'loss' : ["deviance"],
              'n_estimators' : [100,200,300],
              'learning_rate': [0.1, 0.05, 0.01],
              'max_depth': [4, 8],
              'min_samples_leaf': [100,150],
              'max_features': [0.3, 0.1] 
              }

gsGBC = GridSearchCV(GBC,param_grid = gb_param_grid, cv=kfold, scoring="accuracy", verbose = 1)

gsGBC.fit(X_train,y_train)

gb_best = gsGBC.best_estimator_

gsGBC.best_score_

# SVM 
SVMC = SVC(probability=True)
svc_param_grid = {'kernel': ['rbf','linear'], 
                  'gamma': [ 0.01, 0.1, 1],
                  'C': [1, 10, 100, 1000]}

gsSVMC = GridSearchCV(SVMC,param_grid = svc_param_grid, cv=kfold, scoring="accuracy", verbose = 1)

gsSVMC.fit(X_train,y_train)

svm_best = gsSVMC.best_estimator_

gsSVMC.best_score_

#Logistic Regression
LR = LogisticRegression()
gb_param_grid = {'penalty' : ['l1', 'l2', 'elasticnet', 'none'],
              'max_iter' : [10,100,200],
              'solver': ['newton-cg', 'lbfgs', 'liblinear', 'sag', 'saga']
              }
gsLR = GridSearchCV(LR,param_grid = gb_param_grid, cv=kfold, scoring="accuracy", verbose = 1)
gsLR.fit(X_train,y_train)
lr_best = gsLR.best_estimator_

gsLR.best_score_

votingC = VotingClassifier(estimators=[('rfc', rf_best),('svmc', svm_best),
                                       ('adac',ada_best),('gbc',gb_best),('lrc',lr_best)], voting='soft')

votingC = votingC.fit(X_train, y_train)
votingC.score(X_test,y_test)

sklearn_stacking =StackingClassifier(estimators=[('rfc', rf_best),('svc', svm_best),
                                       ('adac',ada_best),('gbc',gb_best),('lrc',lr_best)])
sklearn_stacking = sklearn_stacking.fit(X_train, y_train)
sklearn_stacking.score(X_test,y_test)

from sklearn.ensemble import BaggingClassifier
clf = BaggingClassifier(base_estimator=[('rfc', rf_best),
                                       ('adac',ada_best),('gbc',gb_best)])
clf = clf.fit(X_train, y_train)
clf.score(X_test,y_test)

def get_out_of_fold(clf, x_train, y_train, x_test,kf,NFOLDS):
    oof_train = np.zeros((x_train.shape[0],))
    oof_test = np.zeros((x_test.shape[0],))
    oof_test_skf = np.empty((NFOLDS, x_test.shape[0]))

    for i, (train_index, validation_index) in enumerate(kf.split(x_train,y_train)):
        x_tr = x_train[train_index]
        y_tr = y_train[train_index]
        x_va = x_train[validation_index]

        clf.fit(x_tr, y_tr)

        oof_train[validation_index] = clf.predict(x_va)
        oof_test_skf[i, :] = clf.predict(x_test)
        
    oof_test[:] = oof_test_skf.mean(axis=0)
    
    return oof_train.reshape(-1, 1), oof_test.reshape(-1, 1)

NFOLDS=5
svc_oof_train, svc_oof_test = get_out_of_fold(svm_best,X_train.values, y_train.values, X_test.values,kfold,NFOLDS) # Support Vector Classifier
rf_oof_train, rf_oof_test = get_out_of_fold(rf_best,X_train.values, y_train.values, X_test.values,kfold,NFOLDS) # Random Forest
ada_oof_train, ada_oof_test = get_out_of_fold(ada_best, X_train.values, y_train.values, X_test.values,kfold,NFOLDS) # AdaBoost 
gb_oof_train, gb_oof_test = get_out_of_fold(gb_best,X_train.values, y_train.values, X_test.values,kfold,NFOLDS) # Gradient Boost
lr_oof_train, lr_oof_test = get_out_of_fold(lr_best,X_train.values, y_train.values, X_test.values,kfold,NFOLDS) # Logistic Regression

#First level output as new features
base_predictions_train = pd.DataFrame( {'RandomForest': rf_oof_train.ravel(),
     'AdaBoost': ada_oof_train.ravel(),
      'GradientBoost': gb_oof_train.ravel(),
      'SVM': svc_oof_train.ravel(),
      'LR': lr_oof_train.ravel()                                 
    })
base_predictions_train.head()

#concatenate the predictions from all the models, creating the new train data
x_train = np.concatenate(( rf_oof_train, ada_oof_train, gb_oof_train, svc_oof_train,lr_oof_train), axis=1)
x_test = np.concatenate(( rf_oof_test, ada_oof_test, gb_oof_test, svc_oof_test, lr_oof_test), axis=1)

rfs=rf_best.fit(x_train, y_train)
predictions = rfs.predict(x_test)
predictions #ERRORRR

votingC = VotingClassifier(estimators=[('rfc', rf_best),('svc', svm_best),
                                       ('ada',ada_best),('gbc',gb_best),('lrc',lr_best)], voting='hard')

votingC = votingC.fit(x_train, y_train)
votingC.score(x_test,y_test)